/* Header for Services list handler.
 *
 * Epona (c) 2000 PegSoft
 * Contact us at epona@pegirc.com
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */
 
#ifndef SLIST_H
#define SLIST_H
 
typedef struct slist_ SList;

struct slist_ {
	void **list;
	int16 count;		/* Total entries of the list */
	int16 capacity; 	/* Capacity of the list */
	int16 limit;		/* Maximum possible entries on the list */
	void *data;			/* User data; not used in any way by the lists functions. */
	
	void (*freeitem) (SList *slist, void *item);	/* Called when an item is removed */
};

#define SLIST_DEFAULT_LIMIT 32767

/* Note that number is the index in the array + 1 */
typedef int (*slist_enumcb_t) (SList *slist, int number, void *item, va_list args);
/* Callback to know whether we can delete the entry. */
typedef int (*slist_delcheckcb_t) (SList *slist, void *item, va_list args);

/* Functions for global use */
extern int 	slist_add(SList *slist, void *item);
extern void slist_clear(SList *slist, int free);
extern int 	slist_delete(SList *slist, int index);
extern int 	slist_delete_range(SList *slist, char *range, slist_delcheckcb_t cb, ...);
extern int 	slist_enum(SList *slist, char *range, slist_enumcb_t cb, ...);
extern int 	slist_indexof(SList *slist, void *item);
extern void slist_init(SList *slist);
extern void slist_pack(SList *slist);
extern int 	slist_remove(SList *slist, void *item);
extern int 	slist_setcapacity(SList *slist, int16 capacity);

#endif /* SLIST_H */